#include "mainwindow.h"

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
{
    this->resize(800, 600);

    file_dialog = new QFileDialog(this);
    file_dialog->setFileMode(QFileDialog::ExistingFile);
    file_dialog->setViewMode(QFileDialog::Detail);
    file_dialog->setNameFilter("XML (*.xml)");

    menu = new QMenu(tr("Меню"), this);

    auto exit_action = make_action(tr("Выход"), tr("Закрыть программу"), SLOT(on_exit_action_clicked()));
    auto fullscreen_action = make_action(tr("Во весь экран"), tr("Открыть окно во весь экран/Выйти из полноэкранного режима"),
                                         SLOT(on_fullscreen_action_clicked()));
    exit_action->setShortcut(Qt::Key_Escape);
    fullscreen_action->setShortcut(Qt::Key_F11);
    exit_action->setShortcutContext(Qt::ApplicationShortcut);
    fullscreen_action->setShortcutContext(Qt::ApplicationShortcut);

    menu->addAction(exit_action);
    menu->addAction(fullscreen_action);
    menu->addAction(make_action(tr("Открыть"), tr("Загрузка xml-файла данных"), SLOT(on_open_action_clicked())));
    menu->addAction(make_action(tr("Закрыть всё"), tr("Очистить область вывода данных"), SLOT(on_close_action_clicked())));

    this->statusBar();

    xml_view = new QTreeView(this);
    xml_view->header()->setStretchLastSection(false);
    xml_view->header()->setSectionResizeMode(QHeaderView::Stretch);

    xml_view->setFont(QFont("Calibri", 12));
    menuBar()->setFont(QFont("Calibri", 12));
    statusBar()->setFont(QFont("Calibri", 11));

    this->menuBar()->addMenu(menu);

    this->setCentralWidget(xml_view);
}

MainWindow::~MainWindow()
{
}

QAction* MainWindow::make_action(const QString& p_name, const QString& p_tip, const char* p_slot){
    QAction* action = new QAction(p_name, menu);
    action->setStatusTip(p_tip);
    connect(action, SIGNAL(triggered()), this, p_slot);
    action->setFont(QFont("Calibri", 10));
    return action;
};

void MainWindow::on_open_action_clicked(){
    if(file_dialog->exec()){
        QFile xml_file(file_dialog->selectedFiles().at(0));
        if (!xml_file.exists() || !xml_file.open(QFile::ReadOnly | QFile::Text)) {
            statusBar()->showMessage(tr("Не удалось открыть файл"), 10);
            return;
        }
        QDomDocument doc;
        doc.setContent(&xml_file);
        auto model = new XmlParsingModel(doc, this);
        xml_view->setModel(model);
    }
}

void MainWindow::on_close_action_clicked(){
    xml_view->setModel(nullptr);
}

void MainWindow::on_exit_action_clicked(){
    this->close();
}

void MainWindow::on_fullscreen_action_clicked()
{
    if(isFullScreen())
        showNormal();
    else
        showFullScreen();
}

