#include "xmlparsingmodel.h"

XmlParsingItem::XmlParsingItem(const QDomNode &node, int row, XmlParsingItem *parent)
    : dom_node(node),
      parent_item(parent),
      row_number(row)
{}

XmlParsingItem *XmlParsingItem::parent()
{
    return parent_item;
}

int XmlParsingItem::row() const
{
    return row_number;
}

QDomNode XmlParsingItem::node() const
{
    return dom_node;
}

XmlParsingItem *XmlParsingItem::child(int i)
{
    XmlParsingItem *child_item = child_items.value(i);
    if (child_item)
        return child_item;

    if (i >= 0 && i < dom_node.childNodes().count()) {
        QDomNode childNode = dom_node.childNodes().item(i);
        child_item = new XmlParsingItem(childNode, i, this);
        child_items[i] = child_item;
    }
    return child_item;
}

XmlParsingItem::~XmlParsingItem()
{
    qDeleteAll(child_items);
}

XmlParsingModel::XmlParsingModel(const QDomDocument &document, QObject *parent)
    : QAbstractItemModel(parent),
      xml_document(document),
      root_item(new XmlParsingItem(xml_document, 0))
{
}

XmlParsingModel::~XmlParsingModel()
{
    delete root_item;
}

int XmlParsingModel::columnCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent);
    return 3;
}

Qt::ItemFlags XmlParsingModel::flags(const QModelIndex &index) const
{
    if (!index.isValid())
        return Qt::NoItemFlags;

    return QAbstractItemModel::flags(index);
}

QVariant XmlParsingModel::headerData(int section, Qt::Orientation orientation,
                              int role) const
{
    if (orientation == Qt::Horizontal && role == Qt::DisplayRole) {
        switch (section) {
            case 0:
                return tr("Название");
            case 1:
                return tr("Аттрибуты");
            case 2:
                return tr("Текст");
            default:
                break;
        }
    }
    return QVariant();
}

QModelIndex XmlParsingModel::index(int row, int column, const QModelIndex &parent) const
{
    if (!hasIndex(row, column, parent))
        return QModelIndex();

    XmlParsingItem *parent_item;

    if (!parent.isValid())
        parent_item = root_item;
    else
        parent_item = static_cast<XmlParsingItem*>(parent.internalPointer());

    XmlParsingItem *child_item = parent_item->child(row);
    if (child_item)
        return createIndex(row, column, child_item);
    return QModelIndex();
}

int XmlParsingModel::rowCount(const QModelIndex &parent) const
{
    if (parent.column() > 0)
        return 0;

    XmlParsingItem *parent_item;

    if (!parent.isValid())
        parent_item = root_item;
    else
        parent_item = static_cast<XmlParsingItem*>(parent.internalPointer());

    return parent_item->node().childNodes().count();
}

QModelIndex XmlParsingModel::parent(const QModelIndex &child) const
{
    if (!child.isValid())
        return QModelIndex();

    XmlParsingItem *child_item = static_cast<XmlParsingItem*>(child.internalPointer());
    XmlParsingItem *parent_item = child_item->parent();

    if (!parent_item || parent_item == root_item)
        return QModelIndex();

    return createIndex(parent_item->row(), 0, parent_item);
}

QVariant XmlParsingModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
        return QVariant();
    if(role == Qt::SizeHintRole)
        return QSize(40, 30);
    if (role != Qt::DisplayRole)
        return QVariant();

    const XmlParsingItem *item = static_cast<XmlParsingItem*>(index.internalPointer());

    const QDomNode node = item->node();

    switch (index.column()) {
        case 0:
            return node.nodeName();
        case 1:
        {
            const QDomNamedNodeMap attribute_map = node.attributes();
            QStringList attributes;
            for (int i = 0; i < attribute_map.count(); ++i) {
                QDomNode attribute = attribute_map.item(i);
                attributes << attribute.nodeName() + "=\""
                              + attribute.nodeValue() + '\"';
            }
            return attributes.join(' ');
        }
        case 2:
            return node.nodeValue().split('\n').join(' ');
        default:
            break;
    }
    return QVariant();
}
