#ifndef XMLPARSINGMODEL_H
#define XMLPARSINGMODEL_H

#include <QAbstractItemModel>
#include <QDomDocument>
#include <QDomElement>
#include <QStandardItem>
#include <QFile>

class XmlParsingItem
{
public:
    XmlParsingItem(const QDomNode &node, int row, XmlParsingItem *parent = nullptr);
    ~XmlParsingItem();
    XmlParsingItem *child(int i);
    XmlParsingItem *parent();
    QDomNode node() const;
    int row() const;

private:
    QDomNode dom_node;
    QHash<int, XmlParsingItem *> child_items;
    XmlParsingItem *parent_item;
    int row_number;
};

class XmlParsingModel : public QAbstractItemModel
{
    Q_OBJECT

public:
    explicit XmlParsingModel(const QDomDocument &document, QObject *parent = nullptr);
    ~XmlParsingModel();

    QVariant data(const QModelIndex &index, int role) const override;
    Qt::ItemFlags flags(const QModelIndex &index) const override;
    QVariant headerData(int section, Qt::Orientation orientation,
                        int role = Qt::DisplayRole) const override;
    QModelIndex index(int row, int column,
                      const QModelIndex &parent = QModelIndex()) const override;
    QModelIndex parent(const QModelIndex &child) const override;
    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    int columnCount(const QModelIndex &parent = QModelIndex()) const override;

private:
    QDomDocument xml_document;
    XmlParsingItem *root_item;
};

#endif // XMLPARSINGMODEL_H
